/*********************************************************************************
 *      Copyright:  (C) 2024 Avnet. All Rights Reserved.
 *         Author:  Thomas Nizan <tnizan@witekio.com>
 *
 *    Description:  This program is used to program the Eeprom 
 *                  attached to the IDT8T49N241.
 *
 ********************************************************************************/

#include "at24aa025.h"
#include "xil_printf.h"

static void displayProgramDescription(void) {
    xil_printf("\r\n");
    xil_printf("*********************************************************\r\n");
    xil_printf("*          EEPROM Configuration Tool for 8T49N241       *\r\n");
    xil_printf("*********************************************************\r\n");
    xil_printf("* This program is designed to configure the AT24AA025    *\r\n");
    xil_printf("* EEPROM used in the TRIA Aub15p board to set up the     *\r\n");
    xil_printf("* 8T49N241 clock generator. The EEPROM stores critical   *\r\n");
    xil_printf("* configuration data that defines the clock output       *\r\n");
    xil_printf("* behavior of the 8T49N241.                              *\r\n");
    xil_printf("*                                                       *\r\n");
    xil_printf("* Before writing, the program will perform a full dump   *\r\n");
    xil_printf("* of the current EEPROM contents. This allows you to     *\r\n");
    xil_printf("* review and save the current configuration for          *\r\n");
    xil_printf("* verification and record-keeping purposes.              *\r\n");
    xil_printf("*                                                       *\r\n");
    xil_printf("* You will be prompted to confirm the write operation    *\r\n");
    xil_printf("* to prevent accidental overwrites.                     *\r\n");
    xil_printf("*********************************************************\r\n");
    xil_printf("\r\n");
}


int main(void)
{
	int status;

    displayProgramDescription();

	status = at24aa025_config_dump();
    if ( status != 0 )
    {
        xil_printf("AT24AA025 Config Dump failed");
	    return 1;
    }

    status = at24aa025_config_write_operation();
    if ( status != 0 )
    {
        xil_printf("AT24AA025 Config Write failed");
	    return 1;
    }


	return 0;
}
